module create_pef

  use sep
  use helix
  use createhelixmod
  use bound
  use misinput
  use pef
  use compress
  use print
  use sep3d_struct_mod
  type(filter)                       :: aa   

  contains

  logical function pef_create()
 
  integer, dimension (:), allocatable :: n, n0, a, center, gap
  integer                             :: dim, n123, i, niter, na
  real,    dimension (:), pointer     :: dd, kk
  character (len = 6)                 :: varname
  logical :: do_bound
  type(sep3d) :: resid_s, filt_s
  integer :: ind
  integer, external :: auxin

  ind=0
  ind=auxin('resid')
  if(-1==ind) call seperr('residual file is not provided')
  call auxclose('resid')

  call init_sep3d('resid', resid_s, 'INPUT')

  dim=sep3d_ndims(resid_s)
  if(dim>1) call seperr('Data used to estimate PEF can not have dim more than 1')
  allocate (n(dim), n0 (dim), center (dim), a (dim), gap (dim))
  n=resid_s%n

  call init_sep3d(filt_s,'OUTPUT','FLOAT','REGULAR',ctag='filt_pef')
  call axis_allocate(filt_s,1)  ! The estimated filter is 1D
  call from_param ("peflen", a)      ! a is the lengh of the filter
  filt_s%n(1)=a(1)-1 

!!! the filt file doesn't store the coef 1 and its lag

  call sep3d_set_sep3d(filt_s)
  call sep3d_write_description('filt_pef',filt_s)

  center = 1

  !!where (a (2:) > 1) center (:dim-1) = a(:dim-1)/2 + 1   

  call from_param ("center", center, center)
  call from_param ("na", na, 0)

  if (na == 0) then
     gap = 0
     call from_param ("gap", gap, gap)

     !!! allocate PEF, center: location of coef 1; a:size of filter; n:size of data

     aa = createhelix( n, center, gap, a) ! allocate PEF
     n0 = n
  else
     call allocatehelix (aa, na)
     call from_param ("lag", aa%lag)
     !! must provide aa%lag if na=0
     call from_param ("n", n0)
  end if

  n123 = product (n)
  allocate (dd (n123), kk (n123)) 

  call sreed('resid',dd,4*size(dd))


  if (exist_file ("maskin")) then
     !call sep_read (kk, "maskin")
  else
     kk = 1.
  end if

  call from_param("do_bound",do_bound,.true.)
  if(do_bound) call boundn( n0, n, a, aa) 
  
  call find_mask( (kk /= 0.), aa)   ! missing data

  if (exist_file ("maskout")) then
     !where (aa%mis) 
     !   kk = 0.
     !elsewhere
     !   kk = 1.
     !end where
     !do i=1, dim
     !   call to_history (axisname(i),n(i),"maskout")
     !end do
     !call to_history ("esize",4,"maskout")
     !call sep_write (kk, "maskout")
  end if

  call from_param( "niter", niter, size(aa%lag)*2 )

  !!! Estimate PEF aa, dd is the data from history file used to evaluate filter
  
  call find_pef ( dd, aa, niter)         ! estimate aa
  call resize( aa,.0001)                 ! eliminate zeroes
  call printn( n, center, a, aa)         ! print filter

  !!!write(0,*) 'test', aa%flt, aa%lag
  !!!write(0,*) 'aaa', size(aa%lag), size(aa%flt)

  call srite('filt_pef',aa%flt,4*size(aa%flt))

  call to_history ("lag", aa%lag, 'filt_pef')

 
  deallocate (dd, kk)
  deallocate (n, n0, center, a, gap)

  call  sep3df_delete(filt_s,'filt_pef')
  call  sep3df_delete(resid_s,'resid')

  pef_create=.true.
  end function pef_create

!------------------------------

  logical function return_pef_size(nnn)
  integer :: nnn

  nnn=size(aa%flt)
  return_pef_size=.true.
  end function return_pef_size

!------------------------------

  logical function return_pef(kk) 
     type(filter) :: kk

    kk%flt=aa%flt
    kk%lag=aa%lag
  return_pef=.true.
  end function return_pef


end  module create_pef
